
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

MODULE atmosMOD
	
	use rulesMOD, only: &
	P_dateLength,       &
	P_dirLength,        &
	P_wordLength,       &
	P_atmosAscvar,      &
	P_atmosAscName,     &
	P_debugMode,        &
	P_unit	
	
	USE utilsMOD, only: &
	tArg
	
	
	IMPLICIT NONE
	
	PRIVATE

	TYPE :: tAtmos
		
		CHARACTER(LEN=P_dateLength)                            :: date
		INTEGER                                                :: nx,     &
			                                                  ny,     &
			                                                  nz,     &
			                                                  nv
		REAL                                                   :: lat(2), &
			                                                  lon(2), &
			                                                  xs,     &
			                                                  ys
		CHARACTER(LEN=P_wordLength), ALLOCATABLE, DIMENSION(:) :: varNames	   
		REAL, ALLOCATABLE, DIMENSION(:)                        :: levels
		REAL, ALLOCATABLE, DIMENSION(:,:,:,:)                  :: values
		
	END TYPE tAtmos	

	PUBLIC :: tAtmos
	PUBLIC :: open_atmos
	PUBLIC :: release_atmos
	
	CONTAINS

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
	SUBROUTINE open_atmos(iArg, currDate, dpAtmos, ret)

		TYPE(tArg), INTENT(IN)       :: iArg
		CHARACTER(LEN=*), INTENT(IN) :: currDate
		TYPE(tAtmos), INTENT(OUT)    :: dpAtmos
		INTEGER, INTENT(OUT)         :: ret

		LOGICAL                    :: fileok
		CHARACTER(LEN=P_dirLength) :: file

			
			SELECT CASE(iArg%atmosType)
				CASE(0) !CPTEC/RALPH2/ECMWF - ASC TYPE
					
					CALL create_filename_dpasc(file, currDate, iArg)
					
					INQUIRE(FILE=TRIM(iArg%atmosInputDir)//TRIM(file), EXIST=fileok)
					
					IF(.NOT. fileok)THEN
						IF(P_debugMode .GT. 0) CALL message_output('@> ATMOS DATA FOR : '//currDate//' NOT FOUND!')						
						ret = -1
						RETURN
					END IF
					
					CALL open_atmos_dpasc(TRIM(iArg%atmosInputDir)//TRIM(file), currDate, dpAtmos, ret)
										
				CASE DEFAULT
					
					ret = -1
			END SELECT
			
	END SUBROUTINE open_atmos

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
	SUBROUTINE create_filename_dpasc(file, currDate, iArg)
		
		CHARACTER(LEN=*), INTENT(OUT) :: file
		TYPE(tArg), INTENT(IN)        :: iArg
		CHARACTER(LEN=*), INTENT(IN)  :: currDate
		
			file = TRIM(iArg%atmosPrefix)//                                                          &
				    currDate(1:4)//'-'//currDate(5:6)//'-'//currDate(7:8)//'-'//currDate(9:10)// &
			    	    TRIM(iArg%atmosSufix)
							
	END SUBROUTINE create_filename_dpasc

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE open_atmos_dpasc(file, currDate, dpAtmos, ret)
	
		CHARACTER(LEN=*), INTENT(IN) :: file, &
						currDate		
		TYPE(tAtmos), INTENT(OUT)    :: dpAtmos
		INTEGER, INTENT(OUT)         :: ret
		
		CHARACTER(LEN=P_wordLength) :: msgOut(3)
		INTEGER :: mark,    &
		           version, &
			   dproj,   &
			   i,       &
			   j,       &
			   z,       &
			   v,       &
			   itinc
			   
		INTEGER :: year,  &
		           month, &
			   day,   &
			   hour
			   
		REAL    :: neLat, neLon, cntLat, cntLon, secLat
			
			IF(P_debugMode .GT. 0) CALL message_output('@@@>>>>>>>>>>>>>>>>>>>> OPENING ATMOS - DP ASC FILE : '//TRIM(file))
			
			   	
			OPEN(UNIT=P_unit, FILE=TRIM(file), STATUS='OLD')
			READ(P_unit, *) mark, version
			
			!BRAMS ORIGINAL METHOD
						
			IF(mark .NE. 999999)THEN
				version = 1
			ELSE IF(mark .EQ. 999999)THEN
				version = 2
			END IF
			
			SELECT CASE(version)
			CASE(1)
				!CPTEC
				REWIND(UNIT=P_unit)
				READ(P_unit, *) year, month, day, hour,                           &
			                  	   dpAtmos%nz, dpAtmos%nx, dpAtmos%ny,                    &
						   dpAtmos%lon(1), dpAtmos%lat(1), dpAtmos%xs, dpAtmos%ys
				
				ALLOCATE(dpAtmos%levels(dpAtmos%nz))
				
				READ(P_unit, *)dpAtmos%levels
				
				dproj = 1
			CASE(2)
				!RALPH2/ECMWF	
				READ(P_unit, *) year, month, day, hour, itinc, dpAtmos%nz, dpAtmos%nx, dpAtmos%ny
				READ(P_unit, *) dproj,  dpAtmos%xs, dpAtmos%ys, dpAtmos%lat(1), dpAtmos%lon(1),   &
				neLat, neLon, cntLat, cntLon, secLat
				
				ALLOCATE(dpAtmos%levels(dpAtmos%nz))
				
				READ(P_unit, *)dproj, dpAtmos%levels
							
				!PRINT*, 'HERE'
				!STOP
				!READ(10,*) DP%year, DP%month, DP%day, DP%hour, DP%itinc, DP%nz, &
                           	!	   DP%nx, DP%ny
                		!READ(10,*) dproj, DP%xs, DP%ys, DP%swlat, DP%swlon, DP%nelat, &
                           	!	DP%nelon, DP%cntlat, DP%cntlon, DP%secondlat
                		!ALLOCATE(DP%lvl(DP%nz))
                		!READ(10,*) tmp, DP%lvl

			CASE DEFAULT
			
				PRINT*, 'ATMOS DATA FORMAT NOT DETECTED'
				ret = -1
				RETURN
			
			END SELECT
			
			dpAtmos%nv = P_atmosAscvar
			
			ALLOCATE(dpAtmos%values(dpAtmos%nx, dpAtmos%ny, dpAtmos%nz, P_atmosAscvar))
			
			DO z = 1, dpAtmos%nz
				DO v = 1, P_atmosAscvar
					READ(P_unit,*) ((dpAtmos%values(i,j,z,v),  i = 1, dpAtmos%nx), j = 1, dpAtmos%ny)
				END DO
			END DO				
	
			CLOSE(UNIT=P_unit)
		
			dpAtmos%lat(2) = dpAtmos%lat(1) + (dpAtmos%ys*dpAtmos%ny)
			dpAtmos%lon(2) = dpAtmos%lon(1) + (dpAtmos%xs*dpAtmos%nx)
			
			ALLOCATE(dpAtmos%varNames(P_atmosAscvar))
			
			dpAtmos%date     = currDate
			dpAtmos%varNames = P_atmosAscName
			
			IF(P_debugMode .GT. 1)THEN
				
				CALL message_output('--- DEBUGGING ATMOS TO: '//currDate)
				
				WRITE(msgOut(1), '(I5.5)') dpAtmos%nx
				WRITE(msgOut(2), '(I5.5)') dpAtmos%ny
				WRITE(msgOut(3), '(I5.5)') dpAtmos%nz
				
				CALL message_output('NX          : '//TRIM(msgOut(1)))
				CALL message_output('NY          : '//TRIM(msgOut(2)))
				CALL message_output('NZ          : '//TRIM(msgOut(3)))
				
				WRITE(msgOut(1), '(F9.3)') dpAtmos%lat(1)
				WRITE(msgOut(2), '(F9.3)') dpAtmos%lat(2)
				
				CALL message_output('INITIAL LAT : '//TRIM(msgOut(1)))
				CALL message_output('FINAL   LAT : '//TRIM(msgOut(2)))
				
				WRITE(msgOut(1), '(F9.3)') dpAtmos%lon(1)
				WRITE(msgOut(2), '(F9.3)') dpAtmos%lon(2)
				
				CALL message_output('INITIAL LON : '//TRIM(msgOut(1)))
				CALL message_output('FINAL   LON : '//TRIM(msgOut(2)))
				
				DO i = 1, P_atmosAscvar					
					
					CALL message_output('VARIABLE NAME: '//P_atmosAscName(i))	
					
					DO j = 1, dpAtmos%nz
						WRITE(msgOut(1), '(F9.3)') dpAtmos%levels(j)						
						WRITE(msgOut(2), '(F9.3)') MINVAL(dpAtmos%values(:,:,j,i))	
						WRITE(msgOut(3), '(F9.3)') MAXVAL(dpAtmos%values(:,:,j,i))
						CALL message_output(TRIM(msgOut(1))//' min: '//TRIM(msgOut(2))//' max: '//TRIM(msgOut(3)) )
					END DO			
				END DO
			END IF
			
			ret = 0
		
	END SUBROUTINE open_atmos_dpasc
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE release_atmos(dpAtmos)
		
		TYPE(tAtmos), INTENT(INOUT)    :: dpAtmos
		
			IF(ALLOCATED(dpAtmos%levels))DEALLOCATE(dpAtmos%levels)
			IF(ALLOCATED(dpAtmos%values))DEALLOCATE(dpAtmos%values)
			IF(ALLOCATED(dpAtmos%varNames))DEALLOCATE(dpAtmos%varNames)
	
	END SUBROUTINE release_atmos

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

END MODULE atmosMOD

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
