!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

MODULE icMOD

	USE rulesMOD, only: &
	P_dirLength,        &
	P_undef,            &
	P_unit
	
	USE atmosMOD, only: &
	tAtmos
	
	USE chemMOD, only: &
	tChem
	
	USE utilsMOD, only: &
	datec2datei
	
	IMPLICIT NONE
	
	PRIVATE
	
	PUBLIC :: write_debug_bin
	PUBLIC :: ic_out

	CONTAINS
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
	SUBROUTINE create_filename_out(file, currDate, outPrefix, outSufix)
		
		CHARACTER(LEN=*), INTENT(OUT) :: file
		CHARACTER(LEN=*), INTENT(IN)  :: currDate,  &
						 outPrefix, &
						 outSufix
		
			file = TRIM(outPrefix)//                                                                 &
				    currDate(1:4)//'-'//currDate(5:6)//'-'//currDate(7:8)//'-'//currDate(9:10)// &
			    	    TRIM(outSufix)
				
	END SUBROUTINE create_filename_out
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
	SUBROUTINE ic_out(dpAtmos, dpChem, mergeChem, outDir, outPrefix, outSufix, vfm)
	
		TYPE(tAtmos), INTENT(IN)     :: dpAtmos
		TYPE(tChem), INTENT(IN)      :: dpChem
		LOGICAL, INTENT(IN)          :: mergeChem
		CHARACTER(LEN=*), INTENT(IN) :: outDir, &
						outPrefix, &
						outSufix
		INTEGER, INTENT(IN)          :: vfm        		
		
		CHARACTER(LEN=P_dirLength)        :: file
		INTEGER                           :: year,     &
		           		             month,    &
			   		             day,      &
			   		             hour
		INTEGER                           :: i,        &
			                             j,        &
			                             z,        &
			                             v
		REAL, ALLOCATABLE, DIMENSION(:,:) :: dummy
					       	
			CALL create_filename_out(file, dpAtmos%date, outPrefix, outSufix)
		
		
			CALL datec2datei(hour,day,month,year,dpAtmos%date)
		
			IF(vfm .EQ. 0)THEN
   
 				OPEN(UNIT = P_unit, FILE= TRIM(outDir)//TRIM(file), STATUS = 'replace') 
 
 				WRITE(P_unit,*) year, month, day, hour, dpAtmos%nz, dpAtmos%nx, &
			   	dpAtmos%ny, dpAtmos%lon(1), dpAtmos%lat(1), dpAtmos%xs, dpAtmos%ys
   
 				WRITE(P_unit,*)dpAtmos%levels

 				IF(mergeChem) WRITE(P_unit,*)TRIM(dpChem%chemMech)
 				
				
				DO z = 1, dpAtmos%nz
					DO v = 1, dpAtmos%nv
						WRITE(P_unit,'(F9.3)') ((dpAtmos%values(i,j,z,v),  i = 1, dpAtmos%nx), j = 1, dpAtmos%ny)						
					END DO
					IF(mergeChem)THEN
						DO v = 1, dpChem%nv	
							WRITE(P_unit,'(F9.3)') ((dpChem%values(i,j,z,v),  i = 1, dpChem%nx), j = 1, dpChem%ny)
						END DO				
					END IF
				END DO				
 				
				CLOSE(P_unit)
		
			ELSE
			
				ALLOCATE(dummy(dpAtmos%nx, dpAtmos%ny)) !Atmos need to have same dimensions as Chem.
				
				OPEN(UNIT = P_unit, FILE= TRIM(outDir)//TRIM(file)//'.vfm', STATUS = 'replace') 
 
 				WRITE(P_unit,*) year, month, day, hour, dpAtmos%nz, dpAtmos%nx, &
			   	dpAtmos%ny, dpAtmos%lon(1), dpAtmos%lat(1), dpAtmos%xs, dpAtmos%ys
   
 				WRITE(P_unit,*)dpAtmos%levels

 				IF(mergeChem) WRITE(P_unit,*)TRIM(dpChem%chemMech)
 				
				
				DO z = 1, dpAtmos%nz
					DO v = 1, dpAtmos%nv
						CALL vforec(P_unit, dpAtmos%values(:,:,z,v),dpAtmos%ny*dpAtmos%nx,24,dummy,'LIN')						
					END DO
					IF(mergeChem)THEN
						DO v = 1, dpChem%nv	
							CALL vforec(P_unit, dpChem%values(:,:,z,v),dpChem%ny*dpChem%nx,24,dummy,'LIN')
						END DO				
					END IF
				END DO				
 				
				DEALLOCATE(dummy)
				
				CLOSE(P_unit)
		
			END IF
		
	
	END SUBROUTINE ic_out	
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
	SUBROUTINE write_debug_bin(dpAtmos, dpChem, mergeChem, outDir)
	
	TYPE(tAtmos), INTENT(IN)     :: dpAtmos
	TYPE(tChem), INTENT(IN)      :: dpChem
	LOGICAL, INTENT(IN)          :: mergeChem
	CHARACTER(LEN=*), INTENT(IN) :: outDir
	
	CHARACTER(LEN=4)   		:: levels
	CHARACTER(LEN=255) 		:: outFile
	INTEGER            		:: s,          &
			     		   aux,        &
	                     		   monthIndex, &
	 	             		   printUnit,  &
					   outByteSize
	CHARACTER(LEN=3), DIMENSION(12) :: cMonths
										
			CALL message_output('--- WRITING DEBUG BINARY FOR: '//dpAtmos%date//'.')
			
			DATA cMonths /'jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug', 'sep', 'oct', 'nov', 'dec'/

			
			WRITE(levels,'(I4)') dpAtmos%nz
			
			READ(dpAtmos%date(5:6),'(I2)')monthIndex
				
			outFile = 'DEBUG_OUT_'//dpAtmos%date
										
			OPEN(P_unit, FILE = TRIM(outDir)//TRIM(outfile)//'.ctl', STATUS = 'REPLACE')
 			
			WRITE(P_unit, '(A)')'dset ^'//TRIM(outFile)//'.gra'
 			WRITE(P_unit, '(A, E)')'undef ', P_undef
 			WRITE(P_unit, '(A)')'title dpAtmos DATA BINARY'
 			WRITE(P_unit, '(A, I4, A, 2F)')'xdef', dpAtmos%nx, ' linear', dpAtmos%lon(1), dpAtmos%xs
 			WRITE(P_unit, '(A, I4, A, 2F)')'ydef', dpAtmos%ny, ' linear', dpAtmos%lat(1), dpAtmos%ys
 			WRITE(P_unit, '(A, I4, A, '//TRIM(levels)//'F)')'zdef', INT(dpAtmos%nz), ' levels', dpAtmos%levels
  			WRITE(P_unit, '(A)')'tdef 1 linear '//dpAtmos%date(9:10)//':00z'// &
				                         dpAtmos%date(7:8)//cMonths(monthIndex)//dpAtmos%date(1:4)//' 01hr'
 			
			IF(mergeChem)THEN	
				
				WRITE(P_unit, '(A, I4)')'vars ', dpAtmos%nv + dpChem%nv
 			ELSE

				WRITE(P_unit, '(A, I4)')'vars ', dpAtmos%nv 
			END IF	
			
			DO s=1, dpAtmos%nv
 				WRITE(P_unit, '(A)')TRIM(dpAtmos%varNames(s))//'	'//TRIM(levels)//' 99    -    [   ]'
 			END DO
			
			IF(mergeChem)THEN
				DO s=1, dpChem%nv
 					WRITE(P_unit, '(A)')TRIM(dpChem%varNames(s))//'	'//TRIM(levels)//' 99    -    [   ]'
 				END DO
			END IF	
 			
			WRITE(P_unit, '(A)')'endvars'
 			CLOSE(P_unit)

			INQUIRE(IOLENGTH=outByteSize)dpAtmos%values(:,:,:,1)				
			
			OPEN (UNIT = P_unit, FILE = TRIM(outDir)//TRIM(outfile)//'.gra', ACCESS = 'DIRECT', &
			       RECL = outByteSize, STATUS = 'REPLACE')
 			
			DO s = 1, dpAtmos%nv
				WRITE(P_unit, rec=s)dpAtmos%values(:,:,:,s)
			END DO		
			
			IF(mergeChem)THEN
				aux = 1
				DO s = 1+dpAtmos%nv, (dpAtmos%nv+dpChem%nv)					
					WRITE(UNIT = P_unit, rec=s)dpChem%values(:,:,:,aux)
					aux = aux + 1
				END DO
			END IF	
 			
			CLOSE(UNIT = P_unit)

	END SUBROUTINE write_debug_bin

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
END MODULE icMOD

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
