MODULE utilsMOD

	USE rulesMOD, only: &
	P_dateLength,       &
	P_dirLength,        &
	P_wordLength

	IMPLICIT NONE
	
	PRIVATE
	
	TYPE :: tArg
	
		LOGICAL                     :: chemMerge,     &
		                               ppbmConvert
		CHARACTER(LEN=P_dateLength) :: dateChar
		CHARACTER(LEN=P_dirLength)  :: atmosInputDir, &
					       chemInputDir,  &
					       outDir
		CHARACTER(LEN=P_wordLength) :: atmosPrefix,   &
					       atmosSufix,    &
					       outPrefix,     &
					       outSufix
		INTEGER                     :: atmosType,     &
					       chemType,      &
					       outType,       &
					       chemProj,      &
					       steps(2),      &
					       date(4)		
	END TYPE tArg
	
	PUBLIC :: tArg
	PUBLIC :: greg2jul
	PUBLIC :: jul2greg
	PUBLIC :: datec2datei
	PUBLIC :: datei2datec
	PUBLIC :: step2date
	PUBLIC :: load_arg
	PUBLIC :: sortZ
	PUBLIC :: interpolationNN
	PUBLIC :: interpolationBL
	
	CONTAINS
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
	SUBROUTINE load_arg(inputFile, iArg, ret)
	
		CHARACTER(LEN=*), INTENT(IN) :: inputFile
		TYPE(tArg), INTENT(OUT)      :: iArg
		INTEGER, INTENT(OUT)         :: ret
		
		
		LOGICAL                     :: chem_merge,    &
			                       chem_ppbmconv, &
					       fileok
		INTEGER                     :: init_year,   &
			                       init_month,  &
			                       init_day,    &
			                       init_hour,   &
			                       step,        &
			                       times,       &
			                       atmos_type,  &
					       chem_type,   &
			                       chem_interp, &
			                       out_type 
		CHARACTER(LEN=P_wordLength) :: atmos_prefix, &
					       atmos_sufix,  &
					       out_prefix,   &
					       out_sufix
		CHARACTER(LEN=P_dirLength)  :: atmos_idir, &
					       chem_idir,  &
					       out_dir

		NAMELIST/ARGS_INPUT/init_year, init_month, init_day, init_hour, step, times, atmos_type, atmos_prefix, &
                                    atmos_sufix, atmos_idir, chem_type, chem_merge, chem_interp, chem_ppbmconv, chem_idir,        &
				    out_type, out_prefix, out_sufix, out_dir
			
			
			INQUIRE(FILE='dprep.inp', EXIST=fileok)
			
			IF(.NOT. fileok)THEN
				ret = -1
				RETURN
			END IF
			
			OPEN(UNIT=100, FILE='dprep.inp', STATUS='OLD')
			READ(100, ARGS_INPUT)
			CLOSE(100)	
			
			iArg%date(1) = init_year
			iArg%date(2) = init_month 
			iArg%date(3) = init_day
			iArg%date(4) = init_hour			
			
			CALL datei2datec(init_hour, init_day, init_month, init_year, iArg%dateChar)
			
			iArg%steps(1) = step
			iArg%steps(2) = times	

	        	iArg%atmosInputDir = atmos_idir
			iArg%chemInputDir  = chem_idir
			iArg%outDir        = out_dir
			iArg%atmosPrefix   = atmos_prefix
			iArg%atmosSufix    = atmos_sufix
			iArg%outPrefix     = out_prefix
			iArg%outSufix      = out_sufix
			iArg%atmosType     = atmos_type 
			iArg%chemType      = chem_type 
			iArg%outType       = out_type
			iArg%chemMerge     = chem_merge				  
			iArg%ppbmConvert   = chem_ppbmconv				   
			iArg%chemProj      = chem_interp
			
			ret = 0				
		
	END SUBROUTINE load_arg	
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE step2date(dateIn, dateOut, step, tStep)
		
		CHARACTER(LEN=*), INTENT(IN)   :: datein
		INTEGER, INTENT(IN)            :: step, &
						  tStep
		CHARACTER(len=P_dateLength), INTENT(OUT) :: dateout
		
		REAL(KIND=8)                   :: jd
		
			CALL greg2jul(datein, jd)
		
			!increase day with actual timestep
			jd = ((step * tStep)/24.) + jd 	
		
			!returns to gregorian format 
			CALL jul2greg(jd, dateout)
					
	END SUBROUTINE
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE greg2jul(gdate, jd)

		CHARACTER(len=*), INTENT(IN) :: gdate
		REAL(KIND=8), INTENT(OUT)    :: jd
		
		INTEGER                      :: h,  &
					        d,  &
						m,  &		
						y 	
        	
			CALL datec2datei(h,d,m,y,gdate)
		
			jd = ( 1461 * ( y + 4800 + ( m - 14 ) / 12 ) ) / 4 +        &
             	     	     ( 367 * ( m - 2 - 12 * ( ( m - 14 ) / 12 ) ) ) / 12 -  &
             	     	     ( 3 * ( ( y + 4900 + ( m - 14 ) / 12 ) / 100 ) ) / 4 + &
             	               d - 32075
			jd = jd + (h/24.)

	END SUBROUTINE greg2jul
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE jul2greg(jd, gdate)

		
		REAL(KIND=8), INTENT(IN)       :: jd
		CHARACTER(len=P_dateLength), INTENT(OUT) :: gdate
		
		INTEGER                        :: l, & 
			                          n, &
						  j, &
						  i, &
						  d, &
						  m, &
						  y, &
						  h
		
		        l = jd + 68569
        		n = ( 4 * l ) / 146097
        		l = l - ( 146097 * n + 3 ) / 4
        		i = ( 4000 * ( l + 1 ) ) / 1461001
        		l = l - ( 1461 * i ) / 4 + 31
        		j = ( 80 * l ) / 2447
        		d = l - ( 2447 * j ) / 80
        		l = j / 11
       			m = j + 2 - ( 12 * l )
        		y = 100 * ( n - 49 ) + i + l
	
			h = ( jd - INT(jd) ) * 24.
		
			CALL DateI2DateC(h,d,m,y,gdate)

	END SUBROUTINE jul2greg
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE datec2datei(h,d,m,y,datein)
	
		CHARACTER(len=*), INTENT(IN) :: datein
		INTEGER, INTENT(OUT)         :: h,  &
					        d,  &
						m,  &		
						y						
			
			READ(datein(1:4), '(i4.4)')y				
			READ(datein(5:6), '(i2.2)')m
			READ(datein(7:8), '(i2.2)')d
			READ(datein(9:10),'(i2.2)')h

	END SUBROUTINE datec2datei
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
	SUBROUTINE datei2datec(h,d,m,y,dateout)
	
		INTEGER, INTENT(IN)            :: h,  &
					          d,  &
						  m,  &		
						  y						
		CHARACTER(len=P_dateLength), INTENT(OUT) :: dateout	
		
			WRITE(dateout(1:4), '(i4.4)')y				
			WRITE(dateout(5:6), '(i2.2)')m
			WRITE(dateout(7:8), '(i2.2)')d
			WRITE(dateout(9:10),'(i2.2)')h
		
	END SUBROUTINE datei2datec
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE sortZ(qtLevels, levels, pLevel, order, outIndex)
	
		INTEGER, INTENT(IN)                    :: qtLevels
		REAL, DIMENSION(qtLevels), INTENT(IN)  :: levels
		REAL, INTENT(IN)                       :: pLevel
		INTEGER, INTENT(IN)                    :: order
		INTEGER, DIMENSION(order), INTENT(OUT) :: outIndex
		
		INTEGER                   :: i,     &
			                     j,     &
			                     minIdx
		REAL                      :: currMin
		REAL, DIMENSION(qtLevels) :: ztemp
	
			ztemp = levels
		
			!A SIMPLE SORT THAT RETURNS THE 'order' INDEXES NEAREST OF 'pLevel'
		
			DO j = 1, order
		
				currMin = ABS(ztemp(1) - pLevel)
				minIdx  = 1
			
				DO i = 2, qtLevels
			
					IF( ABS(ztemp(i) - pLevel) .LE. currMin)THEN
				
						currMin = ABS(ztemp(i) - pLevel)
						minIdx  = i
									
					END IF
				END DO				

				outIndex(j) = minIdx
				ztemp(minIdx) = -9E5
					
			END DO		
	
	END SUBROUTINE sortZ

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE interpolationNN(iValues, iNX, iNY, iNZ, iLat, iLon, iXS, iYS, oValues, oNX, oNY, oNZ, oLat, oLon, oXS, oYS)
	
		INTEGER, INTENT(IN)                         :: iNX,  &
				                               iNY,  &
				                               iNZ,  &
						               oNX,  &
				                               oNY,  &
				                               oNZ
		REAL, DIMENSION(2),INTENT(IN)               :: iLat, &
				                               iLon, &
						               oLat, &
				                               oLon
		REAL, INTENT(IN)                            :: iXS,  &
				                               iYS,  &
							       oXS,  &
				                               oYS
		REAL, DIMENSION(iNX,iNY,iNZ), INTENT(IN)    :: iValues
		REAL, DIMENSION(oNX,oNY,oNZ), INTENT(INOUT) :: oValues
		
		INTEGER, PARAMETER                   :: localUndef = -999
		
		INTEGER                              :: i,           &
							j,           &
							z,           &
							px,          &
							py,          &
							idxLon(1),   &
							idxLat(1),   &
							signx,       &
							signy
		REAL                                 :: lon,         &
							lat,         &
							tmpLat(2),   &
							tmpLon(2)
		REAL, ALLOCATABLE, DIMENSION(:)	     :: latRepro,    &
							lonRepro,    &
							latOrig,     &
							lonOrig
		INTEGER, ALLOCATABLE, DIMENSION(:) :: indexReproX, &
						      indexReproY

			IF(.NOT. ALLOCATED(indexReproX))THEN
				
				ALLOCATE(indexReproX(oNX), indexReproY(oNY))
				ALLOCATE(latRepro(oNY), lonRepro(oNX))
				ALLOCATE(latOrig(iNY), lonOrig(iNX))
				
				indexReproX = localUndef
				indexReproY = localUndef
				
				!CREATES A ORIGINAL DATA LON/LAT VECTOR
				
				DO i = 1, iNX

					IF(iLon(1) .LT. iLon(2))THEN
						lon = iLon(1) + (iXS*(i-1))
					ELSE
						lon = iLon(1) - (iXS*(i-1))						
					END IF
					
					lon = lon - (MIN(1,INT(lon/181))*360)
					lonOrig(i) = lon
					
				END DO
				
				DO j = 1, iNY

					IF(iLat(1) .LT. iLat(2))THEN
						lat = iLat(1) + (iYS*(j-1))
					ELSE
						lat = iLat(1) - (iYS*(j-1))						
					END IF
					
					lat = lat - (MIN(1,INT(lat/91))*180)
					latOrig(j) = lat
					
				END DO				
				
				!FIND MAX/MIN BOUNDS IN ORIG LAT/LON VECTOR
				!TRYING TO AVOID PROBLEMS IN REPROJECTION WHEN THE DIMENSION ENDS IN VALUES GREATER THAN 180 DEGREES.
				!0.......180.....360 => 181.....359.0......180
	
				idxLat = MINLOC(latOrig)
				tmpLat(1) = MINVAL(latOrig)
				tmpLat(2) = MAXVAL(latOrig)
					
				idxLon = MINLOC(lonOrig)
				tmpLon(1) = MINVAL(lonOrig)
				tmpLon(2) = MAXVAL(lonOrig)
									
				signx = 1
				signy = 1
				
				!IF LON(1)/LAT(1) ARE NOT THE LOWEST/LEFT BOUND THEN CHANGE THE DIRECTION
				
				IF(iLon(1) .GT. iLon(2)) signx = -1					
				IF(iLat(1) .GT. iLat(2)) signy = -1
				
				!CREATES A MATRIX THAT DEFINES THE RELATION BETWEEN ORIGINAL AND REPROJECTED DATA. 				
				
				DO i = 1, oNX				
					
					lon = oLon(1) + (oXS*(i-1))
			
					IF(lon .GE. tmpLon(1) .AND. lon .LE. tmpLon(2))THEN
						px = ((lon - tmpLon(1))/iXS)
					ELSE
						px = -1
					END IF
					
					lonRepro(i) = lon

					IF(px .GE. 0)THEN
						px = idxLon(1) + (INT(px) * signx)
						IF(px .GT. iNX) px = MOD(px, iNX)
					END IF
							
					IF(px .GT. 0 .AND. px .LE. iNX)indexReproX(i) = px
				END DO
				
				DO j = 1, oNY				
					
					lat = oLat(1) + (oYS*(j-1))
			
					IF(lat .GE. tmpLat(1) .AND. lat .LE. tmpLat(2))THEN
						py = ((lat - tmpLat(1))/iYS)
					ELSE
						py = -1
					END IF
					
					latRepro(j) = lat

					IF(py .GE. 0)THEN
						py = idxLat(1) + (INT(py) * signy)
						IF(py .GT. iNY) py = MOD(py, iNY)
					END IF
							
					IF(py .GT. 0 .AND. py .LE. iNY)indexReproY(j) = py
				END DO				
				
			END IF			
			
			DO z = 1, oNZ
				DO j = 1, oNY	
					DO i = 1, oNX										
						IF(indexReproX(i) .NE. localUndef .AND. indexReproY(j) .NE. localUndef)&							
							oValues(i,j,z) = iValues(indexReproX(i), indexReproY(j), z)															
					END DO
				END DO
			END DO
						
			IF(ALLOCATED(indexReproX))THEN
				DEALLOCATE(indexReproX, indexReproY)
				DEALLOCATE(latRepro,lonRepro)
				DEALLOCATE(latOrig, lonOrig)
			END IF
	
	END SUBROUTINE interpolationNN

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE interpolationBL(iValues, iNX, iNY, iNZ, iLat, iLon, iXS, iYS, oValues, oNX, oNY, oNZ, oLat, oLon, oXS, oYS)
	
		INTEGER, INTENT(IN)                         :: iNX,  &
				                               iNY,  &
				                               iNZ,  &
						               oNX,  &
				                               oNY,  &
				                               oNZ
		REAL, DIMENSION(2),INTENT(IN)               :: iLat, &
				                               iLon, &
						               oLat, &
				                               oLon
		REAL, INTENT(IN)                            :: iXS,  &
				                               iYS,  &
							       oXS,  &
				                               oYS
		REAL, DIMENSION(iNX,iNY,iNZ), INTENT(IN)    :: iValues
		REAL, DIMENSION(oNX,oNY,oNZ), INTENT(INOUT) :: oValues
		
		INTEGER, PARAMETER                   :: localUndef = -999
		
		INTEGER                              :: i,           &
							j,           &
							z,           &
							px,          &
							py,          &
							idxLon(1),   &
							idxLat(1),   &
							signx,       &
							signy
		REAL                                 :: lon,         &
							lat,         &
							tmpLat(2),   &
							tmpLon(2)
		REAL                                 :: x0, x1, x2, y0, y1, y2, f11, f12, f21, f22, fr1, fr2, p
		REAL, ALLOCATABLE, DIMENSION(:)	     :: latRepro,    &
							lonRepro,    &
							latOrig,     &
							lonOrig
		INTEGER, ALLOCATABLE, DIMENSION(:,:) :: indexReproX, &
						        indexReproY

			IF(.NOT. ALLOCATED(indexReproX))THEN
				
				ALLOCATE(indexReproX(2,oNX), indexReproY(2,oNY))
				ALLOCATE(latRepro(oNY), lonRepro(oNX))
				ALLOCATE(latOrig(iNY), lonOrig(iNX))
				
				indexReproX = localUndef
				indexReproY = localUndef
				
				!CREATES A ORIGINAL DATA LON/LAT VECTOR
				
				DO i = 1, iNX

					IF(iLon(1) .LT. iLon(2))THEN
						lon = iLon(1) + (iXS*(i-1))
					ELSE
						lon = iLon(1) - (iXS*(i-1))						
					END IF
					
					lon = lon - (MIN(1,INT(lon/181))*360)
					lonOrig(i) = lon
					
				END DO
				
				DO j = 1, iNY

					IF(iLat(1) .LT. iLat(2))THEN
						lat = iLat(1) + (iYS*(j-1))
					ELSE
						lat = iLat(1) - (iYS*(j-1))						
					END IF
					
					lat = lat - (MIN(1,INT(lat/91))*180)
					latOrig(j) = lat
					
				END DO				
				
				!FIND MAX/MIN BOUNDS IN ORIG LAT/LON VECTOR
				!TRYING TO AVOID PROBLEMS IN REPROJECTION WHEN THE DIMENSION ENDS IN VALUES GREATER THAN 180 DEGREES.
				!0.......180.....360 => 181.....359.0......180
	
				idxLat = MINLOC(latOrig)
				tmpLat(1) = MINVAL(latOrig)
				tmpLat(2) = MAXVAL(latOrig)
					
				idxLon = MINLOC(lonOrig)
				tmpLon(1) = MINVAL(lonOrig)
				tmpLon(2) = MAXVAL(lonOrig)
									
				signx = 1
				signy = 1
				
				!IF LON(1)/LAT(1) ARE NOT THE LOWEST/LEFT BOUND THEN CHANGE THE DIRECTION
				
				IF(iLon(1) .GT. iLon(2)) signx = -1					
				IF(iLat(1) .GT. iLat(2)) signy = -1
				
				!CREATES A MATRIX THAT DEFINES THE RELATION BETWEEN ORIGINAL AND REPROJECTED DATA. 				
				
				DO i = 1, oNX				
					
					lon = oLon(1) + (oXS*(i-1))
			
					IF(lon .GE. tmpLon(1) .AND. lon .LE. tmpLon(2))THEN
						px = ((lon - tmpLon(1))/iXS)
					ELSE
						px = -1
					END IF
					
					lonRepro(i) = lon

					IF(px .GE. 0)THEN
						px = idxLon(1) + (INT(px) * signx)
						IF(px .GT. iNX) px = MOD(px, iNX)
					END IF
							
					IF(px .GT. 0 .AND. px .LE. iNX)                indexReproX(1,i) = px
					IF((px+signx) .GT. 0 .AND. (px+signx) .LE. iNX)indexReproX(2,i) = px + signx
				END DO
				
				DO j = 1, oNY				
					
					lat = oLat(1) + (oYS*(j-1))
			
					IF(lat .GE. tmpLat(1) .AND. lat .LE. tmpLat(2))THEN
						py = ((lat - tmpLat(1))/iYS)
					ELSE
						py = -1
					END IF
					
					latRepro(j) = lat

					IF(py .GE. 0)THEN
						py = idxLat(1) + (INT(py) * signy)
						IF(py .GT. iNY) py = MOD(py, iNY)
					END IF
					
					IF(py .GT. 0 .AND. py .LE. iNY)                indexReproY(1,j) = py		
					IF((py+signy) .GT. 0 .AND. (py+signy) .LE. iNY)indexReproY(2,j) = py + signy
				END DO				
				
			END IF			
			
			DO z = 1, oNZ
				DO j = 1, oNY
					DO i = 1, oNX					
						
						IF(indexReproX(1,i) .NE. localUndef .AND. indexReproY(1,j) .NE. localUndef)THEN
							
							IF(indexReproX(2,i) .NE. localUndef .AND. indexReproY(2,j) .NE. localUndef)THEN
							
								x0 = lonRepro(i)
								y0 = latRepro(j)
								
								x1 = lonOrig(indexReproX(1,i))
								y1 = latOrig(indexReproY(1,j))
								
								x2 = lonOrig(indexReproX(2,i))
								y2 = latOrig(indexReproY(2,j))
								
								f11 = iValues(indexReproX(1,i), indexReproY(1,j), z)
								f12 = iValues(indexReproX(1,i), indexReproY(2,j), z)
								f21 = iValues(indexReproX(2,i), indexReproY(1,j), z)
								f22 = iValues(indexReproX(2,i), indexReproY(2,j), z)
								
								IF(f11 .NE. localUndef .AND. f12 .NE. localUndef .AND. &
								   f21 .NE. localUndef .AND. f22 .NE. localUndef)THEN
								
									fr1 = ( (((x2-x0)/(x2-x1))*f11) + (((x0-x1)/(x2-x1))*f21) )
  									fr2 = ( (((x2-x0)/(x2-x1))*f12) + (((x0-x1)/(x2-x1))*f22) )
								
									p = ( ((y2-y0)/(y2-y1)*fr1) + ((y0-y1)/(y2-y1)*fr2) ) 
									  
									!NaN CHECK
									IF(p .NE. p) THEN
										oValues(i,j,z) = localUndef
									ELSE
										oValues(i,j,z) = p
									END IF								   
								   
								END IF
							
							ELSE
								!NO NEIGHBOORS AVAILABLE
								oValues(i,j,z) = iValues(indexReproX(1,i), indexReproY(1,j), z)
							END IF
						END IF
					END DO
				END DO
			END DO

			IF(ALLOCATED(indexReproX))THEN
				DEALLOCATE(indexReproX, indexReproY)
				DEALLOCATE(latRepro,lonRepro)
				DEALLOCATE(latOrig, lonOrig)
			END IF
	
	END SUBROUTINE interpolationBL

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

END MODULE utilsMOD
